@@ -11,14 +11,14 @@ use crate::{
 
 cfg_if! {
     if #[cfg(any(all(feature = "f3", not(feature = "f302")), all(feature = "l4", not(feature = "l4x6")), feature = "g4", feature = "h7b3"))] {
-        use crate::pac::dac1 as dac_p;
+        use pac::dac1 as dac_p;
     } else {
-        use crate::pac::dac as dac_p;
+        use pac::dac as dac_p;
     }
 }
 
 #[cfg(feature = "g0")]
-use crate::pac::dma as dma_p;
+use pac::dma as dma_p;
 #[cfg(any(
     feature = "f3",
     feature = "l4",
@@ -27,7 +27,7 @@ use crate::pac::dma as dma_p;
     feature = "wb",
     feature = "wl"
 ))]
-use crate::pac::dma1 as dma_p;
+use pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "f4", feature = "l5")))]
 use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
@@ -164,18 +164,21 @@ where
         // See H743 RM, Table 227 for info on the buffer.
         // todo: Currently at default setting for both channels of external pin with buffer enabled.
         // todo make this customizable
-        let mode = DacMode::NormExternalOnlyBufEn;
-        #[cfg(not(any(
-            feature = "f3",
-            feature = "f4",
-            feature = "l5",
-            feature = "g4",
-            feature = "wl"
-        )))]
-        regs.mcr.modify(|_, w| unsafe {
-            w.mode1().bits(mode as u8);
-            w.mode2().bits(mode as u8)
-        });
+        cfg_if! {
+            if #[cfg(not(any(
+                feature = "f3",
+                feature = "f4",
+                feature = "l5",
+                feature = "g4",
+                feature = "wl"
+            )))] {
+                let mode = DacMode::NormExternalOnlyBufEn;
+                regs.mcr.modify(|_, w| unsafe {
+                    w.mode1().bits(mode as u8);
+                    w.mode2().bits(mode as u8)
+                });
+            }
+        }
 
         Self { regs, bits, vref }
     }
