@@ -13,15 +13,42 @@ use cortex_m::interrupt::free;
 
 use crate::{
     pac::{self, EXTI, RCC},
-    rcc_en_reset,
+    rcc_en_reset, // todo?
 };
 
-#[cfg(not(any(feature = "l5", feature = "g0")))]
-use crate::pac::SYSCFG;
-
 #[cfg(feature = "embedded-hal")]
 use embedded_hal::digital::v2::{InputPin, OutputPin, ToggleableOutputPin};
 
+#[cfg(not(any(
+    feature = "g0",
+    feature = "f4",
+    feature = "l5",
+    feature = "f3",
+    feature = "l4"
+)))]
+use core::ops::Deref;
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::util::DmaPeriph;
+
+#[cfg(feature = "g0")]
+use crate::pac::dma as dma_p;
+#[cfg(any(
+    feature = "f3",
+    feature = "l4",
+    feature = "g4",
+    feature = "h7",
+    feature = "wb",
+    feature = "wl",
+))]
+use crate::pac::dma1 as dma_p;
+
+#[cfg(not(any(feature = "f4", feature = "l5")))]
+use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::dma::DmaInput;
+
 use cfg_if::cfg_if;
 use paste::paste;
 
@@ -65,8 +92,8 @@ pub enum OutputSpeed {
     Low = 0b00,
     Medium = 0b01,
     #[cfg(not(feature = "f3"))]
-    Fast = 0b10,
-    High = 0b11,
+    Fast = 0b10, // Called "High speed" on some families.
+    High = 0b11, // Called "Very high speed" on some families.
 }
 
 #[derive(Copy, Clone)]
@@ -683,10 +710,17 @@ impl Pin {
                             }
                         }
                     }
-                    #[cfg(feature = "l5")] // also for RM0351 L4 variants, which we don't currently support
+                    #[cfg(feature = "l5")]
+                    // also for RM0351 L4 variants, which we don't currently support
                     // L5 RM: "[The IOSV bit] is used to validate the VDDIO2 supply for electrical and logical isolation purpose.
                     // Setting this bit is mandatory to use PG[15:2]."
-                    { unsafe { (*crate::pac::PWR::ptr()).cr2.modify(|_,w| w.iosv().set_bit()); } }
+                    {
+                        unsafe {
+                            (*crate::pac::PWR::ptr())
+                                .cr2
+                                .modify(|_, w| w.iosv().set_bit());
+                        }
+                    }
                 }
                 #[cfg(not(any(
                     feature = "f373",
@@ -1016,6 +1050,128 @@ fn set_state(port: Port, pin: u8, value: PinState) {
     );
 }
 
+/// Clear an EXTI interrupt, lines 0 - 15. Note that this function currently doesn't support
+/// higher extis, but will work for all GPIO interrupts.
+pub fn clear_exti_interrupt(line: u8) {
+    // todo: Macro to avoid DRY?
+    unsafe {
+        cfg_if! {
+            if #[cfg(any(feature = "h747cm4", feature = "h747cm7"))] {
+                (*EXTI::ptr()).c1pr1.modify(|_, w| {
+                    match line {
+                        0 => w.pr0().set_bit(),
+                        1 => w.pr1().set_bit(),
+                        2 => w.pr2().set_bit(),
+                        3 => w.pr3().set_bit(),
+                        4 => w.pr4().set_bit(),
+                        5 => w.pr5().set_bit(),
+                        6 => w.pr6().set_bit(),
+                        7 => w.pr7().set_bit(),
+                        8 => w.pr8().set_bit(),
+                        9 => w.pr9().set_bit(),
+                        10 => w.pr10().set_bit(),
+                        11 => w.pr11().set_bit(),
+                        12 => w.pr12().set_bit(),
+                        13 => w.pr13().set_bit(),
+                        14 => w.pr14().set_bit(),
+                        15 => w.pr15().set_bit(),
+                        _ => panic!(),
+                    }
+                });
+            } else if #[cfg(feature = "h7")] {
+                (*EXTI::ptr()).d3pmr1.modify(|_, w| {
+                    match line {
+                        0 => w.mr0().set_bit(),
+                        1 => w.mr1().set_bit(),
+                        2 => w.mr2().set_bit(),
+                        3 => w.mr3().set_bit(),
+                        4 => w.mr4().set_bit(),
+                        5 => w.mr5().set_bit(),
+                        6 => w.mr6().set_bit(),
+                        7 => w.mr7().set_bit(),
+                        8 => w.mr8().set_bit(),
+                        9 => w.mr9().set_bit(),
+                        10 => w.mr10().set_bit(),
+                        11 => w.mr11().set_bit(),
+                        12 => w.mr12().set_bit(),
+                        13 => w.mr13().set_bit(),
+                        14 => w.mr14().set_bit(),
+                        15 => w.mr15().set_bit(),
+                        _ => panic!(),
+                    }
+                });
+            } else if #[cfg(feature = "l5")] {
+                (*EXTI::ptr()).rpr1.modify(|_, w| {
+                    match line {
+                        0 => w.rpif0().set_bit(),
+                        1 => w.rpif1().set_bit(),
+                        2 => w.rpif2().set_bit(),
+                        3 => w.rpif3().set_bit(),
+                        4 => w.rpif4().set_bit(),
+                        5 => w.rpif5().set_bit(),
+                        6 => w.rpif6().set_bit(),
+                        7 => w.rpif7().set_bit(),
+                        8 => w.rpif8().set_bit(),
+                        9 => w.rpif9().set_bit(),
+                        10 => w.rpif10().set_bit(),
+                        11 => w.rpif11().set_bit(),
+                        12 => w.rpif12().set_bit(),
+                        13 => w.rpif13().set_bit(),
+                        14 => w.rpif14().set_bit(),
+                        15 => w.rpif15().set_bit(),
+                        _ => panic!(),
+                    }
+                });
+            } else if #[cfg(feature = "f4")] {
+                (*EXTI::ptr()).pr.modify(|_, w| {
+                    match line {
+                        0 => w.pr0().set_bit(),
+                        1 => w.pr1().set_bit(),
+                        2 => w.pr2().set_bit(),
+                        3 => w.pr3().set_bit(),
+                        4 => w.pr4().set_bit(),
+                        5 => w.pr5().set_bit(),
+                        6 => w.pr6().set_bit(),
+                        7 => w.pr7().set_bit(),
+                        8 => w.pr8().set_bit(),
+                        9 => w.pr9().set_bit(),
+                        10 => w.pr10().set_bit(),
+                        11 => w.pr11().set_bit(),
+                        12 => w.pr12().set_bit(),
+                        13 => w.pr13().set_bit(),
+                        14 => w.pr14().set_bit(),
+                        15 => w.pr15().set_bit(),
+                        _ => panic!(),
+                    }
+                });
+            }
+            else {
+                (*EXTI::ptr()).pr1.modify(|_, w| {
+                    match line {
+                        0 => w.pif0().set_bit(),
+                        1 => w.pif1().set_bit(),
+                        2 => w.pif2().set_bit(),
+                        3 => w.pif3().set_bit(),
+                        4 => w.pif4().set_bit(),
+                        5 => w.pif5().set_bit(),
+                        6 => w.pif6().set_bit(),
+                        7 => w.pif7().set_bit(),
+                        8 => w.pif8().set_bit(),
+                        9 => w.pif9().set_bit(),
+                        10 => w.pif10().set_bit(),
+                        11 => w.pif11().set_bit(),
+                        12 => w.pif12().set_bit(),
+                        13 => w.pif13().set_bit(),
+                        14 => w.pif14().set_bit(),
+                        15 => w.pif15().set_bit(),
+                        _ => panic!(),
+                    }
+                });
+            }
+        }
+    }
+}
+
 const fn regs(port: Port) -> *const pac::gpioa::RegisterBlock {
     // Note that we use this `const` fn and pointer casting since not all ports actually
     // deref to GPIOA in PAC.
@@ -1074,4 +1230,45 @@ const fn regs(port: Port) -> *const pac::gpioa::RegisterBlock {
         )))]
         Port::H => crate::pac::GPIOH::ptr() as _,
     }
-}
\ No newline at end of file
+}
+
+#[cfg(not(any(
+    feature = "g0",
+    feature = "f4",
+    feature = "l5",
+    feature = "f3",
+    feature = "l4"
+)))]
+/// Write a series of words to the BSRR (atomic output) register. Note that these are direct writes
+/// to the full, 2-sided register - not a series of low/high values.
+pub unsafe fn write_dma<D>(
+    buf: &[u32],
+    port: Port,
+    dma_channel: DmaChannel,
+    channel_cfg: ChannelCfg,
+    dma: &mut Dma<D>,
+) where
+    D: Deref<Target = dma_p::RegisterBlock>,
+{
+    let (ptr, len) = (buf.as_ptr(), buf.len());
+
+    // todo: DMA2 support.
+
+    let periph_addr = &(*(regs(port))).bsrr as *const _ as u32;
+
+    #[cfg(feature = "h7")]
+    let num_data = len as u32;
+    #[cfg(not(feature = "h7"))]
+    let num_data = len as u16;
+
+    dma.cfg_channel(
+        dma_channel,
+        periph_addr,
+        ptr as u32,
+        num_data,
+        dma::Direction::ReadFromMem,
+        dma::DataSize::S32,
+        dma::DataSize::S32,
+        channel_cfg,
+    );
+}
