@@ -11,9 +11,12 @@ use embedded_hal::spi::FullDuplex;
 
 use crate::{
     pac::{self, RCC},
-    util::{DmaPeriph, RccPeriph},
+    util::RccPeriph,
 };
 
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::util::DmaPeriph;
+
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
 #[cfg(any(
@@ -27,7 +30,7 @@ use crate::pac::dma as dma_p;
 use crate::pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "f4", feature = "l5")))]
-use crate::dma::{self, Dma, DmaChannel};
+use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
 
 #[cfg(any(feature = "f3", feature = "l4"))]
 use crate::dma::DmaInput;
@@ -234,7 +237,7 @@ impl SpiMode {
     }
 }
 
-/// Initial configuration data for the SPI peripheral.
+/// Configuration data for SPI.
 pub struct SpiConfig {
     /// SPI mode associated with Polarity and Phase. Defaults to Mode0: Idle low, capture on first transition.
     pub mode: SpiMode,
@@ -269,7 +272,8 @@ pub struct Spi<R> {
 
 impl<R> Spi<R>
 where
-    R: Deref<Target = pac::spi1::RegisterBlock> + DmaPeriph + RccPeriph,
+    // R: Deref<Target = pac::spi1::RegisterBlock> + DmaPeriph + RccPeriph,
+    R: Deref<Target = pac::spi1::RegisterBlock> + RccPeriph,
 {
     /// Initialize an SPI peripheral, including configuration register writes, and enabling and resetting
     /// its RCC peripheral clock.
@@ -512,7 +516,6 @@ where
             if #[cfg(feature = "h7")] {
                 let crce = sr.crce().bit_is_set();
                 let rdy = sr.txp().bit_is_set();
-                let rdy = sr.txp().bit_is_set();
             } else {
                 let crce = sr.crcerr().bit_is_set();
                 let rdy = sr.txe().bit_is_set();
@@ -548,6 +551,7 @@ where
     pub fn write(&mut self, words: &[u8]) -> Result<(), Error> {
         for word in words {
             nb::block!(self.write_one(word.clone()))?;
+            // nb::block!(self.write_one(word))?; // todo: Test this without clone once you have SPI working.
             nb::block!(self.read())?;
         }
 
@@ -558,6 +562,7 @@ where
     /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
     pub fn transfer<'w>(&mut self, words: &'w mut [u8]) -> Result<(), Error> {
         for word in words.iter_mut() {
+            // nb::block!(self.write_one(word))?; // todo: See note above.
             nb::block!(self.write_one(word.clone()))?;
             *word = nb::block!(self.read())?;
         }
@@ -565,16 +570,23 @@ where
         Ok(())
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Transmit data using DMA. See L44 RM, section 40.4.9: Communication using DMA.
     /// Note that the `channel` argument has no effect on F3 and L4.
-    pub unsafe fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
-    where
+    pub unsafe fn write_dma<D>(
+        &mut self,
+        buf: &[u8],
+        channel: DmaChannel,
+        channel_cfg: ChannelCfg,
+        dma: &mut Dma<D>,
+    ) where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         // Static write and read buffers?
         let (ptr, len) = (buf.as_ptr(), buf.len());
 
+        self.regs.cr1.modify(|_, w| w.spe().clear_bit());
+
         // todo: Accept u16 words too.
         // todo: Pri and Circular as args?
 
@@ -601,40 +613,53 @@ where
         #[cfg(not(feature = "h7"))]
         let periph_addr = &self.regs.dr as *const _ as u32;
 
+        #[cfg(feature = "h7")]
+        let num_data = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let num_data = len as u16;
+
         dma.cfg_channel(
             channel,
             periph_addr,
             ptr as u32,
-            len as u16,
+            num_data,
             dma::Direction::ReadFromMem,
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
 
-        // atomic::compiler_fence(Ordering::Release);  // todo ?
-
         // 3. Enable DMA Tx buffer in the TXDMAEN bit in the SPI_CR2 register, if DMA Tx is used.
+        #[cfg(not(feature = "h7"))]
         self.regs.cr2.modify(|_, w| w.txdmaen().set_bit());
+        #[cfg(feature = "h7")]
+        self.regs.cfg1.modify(|_, w| w.txdmaen().set_bit());
 
         // 4. Enable the SPI by setting the SPE bit.
         self.regs.cr1.modify(|_, w| w.spe().set_bit());
-        // (todo: Should be already set. Should we disable it at the top of this fn just in case?)
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Receive data using DMA. See L44 RM, section 40.4.9: Communication using DMA.
-    /// Note taht the `channel` argument has no effect on F3 and L4.
-    pub unsafe fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
-    where
+    /// Note thay the `channel` argument has no effect on F3 and L4.
+    pub unsafe fn read_dma<D>(
+        &mut self,
+        buf: &mut [u8],
+        channel: DmaChannel,
+        channel_cfg: ChannelCfg,
+        dma: &mut Dma<D>,
+    ) where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         // todo: Accept u16 words too.
         let (ptr, len) = (buf.as_mut_ptr(), buf.len());
 
-        // atomic::compiler_fence(Ordering::Release);  // todo ?
+        self.regs.cr1.modify(|_, w| w.spe().clear_bit());
 
+        #[cfg(not(feature = "h7"))]
         self.regs.cr2.modify(|_, w| w.rxdmaen().set_bit());
+        #[cfg(feature = "h7")]
+        self.regs.cfg1.modify(|_, w| w.rxdmaen().set_bit());
 
         #[cfg(any(feature = "f3", feature = "l4"))]
         let channel = R::read_chan();
@@ -646,23 +671,114 @@ where
         #[cfg(not(feature = "h7"))]
         let periph_addr = &self.regs.dr as *const _ as u32;
 
+        #[cfg(feature = "h7")]
+        let num_data = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let num_data = len as u16;
+
         dma.cfg_channel(
             channel,
             periph_addr,
             ptr as u32,
-            len as u16,
+            num_data,
             dma::Direction::ReadFromPeriph,
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
 
         self.regs.cr1.modify(|_, w| w.spe().set_bit());
-
-        // todo: Set rxne or something to start?
     }
 
-    // todo: pub fn transfer_dma()?
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
+    /// Transfer data from DMA; this is the basic reading API, using both write and read transfers:
+    /// It performs a write with register data, and reads to a buffer.
+    pub unsafe fn transfer_dma<D>(
+        &mut self,
+        buf_write: &[u8],
+        buf_read: &mut [u8],
+        channel_write: DmaChannel,
+        channel_read: DmaChannel,
+        channel_cfg_write: ChannelCfg,
+        channel_cfg_read: ChannelCfg,
+        dma: &mut Dma<D>,
+    ) where
+        D: Deref<Target = dma_p::RegisterBlock>,
+    {
+        // todo: Accept u16 words too.
+        let (ptr_write, len_write) = (buf_write.as_ptr(), buf_write.len());
+        let (ptr_read, len_read) = (buf_read.as_mut_ptr(), buf_read.len());
+
+        self.regs.cr1.modify(|_, w| w.spe().clear_bit());
+
+        // todo: DRY here, with `write_dma`, and `read_dma`.
+
+        #[cfg(any(feature = "f3", feature = "l4"))]
+        let channel_write = R::write_chan();
+        #[cfg(feature = "l4")]
+        R::write_sel(dma);
+
+        #[cfg(any(feature = "f3", feature = "l4"))]
+        let channel_read = R::read_chan();
+        #[cfg(feature = "l4")]
+        R::read_sel(dma);
+
+        #[cfg(feature = "h7")]
+        let periph_addr_write = &self.regs.txdr as *const _ as u32;
+        #[cfg(not(feature = "h7"))]
+        let periph_addr_write = &self.regs.dr as *const _ as u32;
+
+        #[cfg(feature = "h7")]
+        let periph_addr_read = &self.regs.rxdr as *const _ as u32;
+        #[cfg(not(feature = "h7"))]
+        let periph_addr_read = &self.regs.dr as *const _ as u32;
+
+        #[cfg(feature = "h7")]
+        let num_data_write = len_write as u32;
+        #[cfg(not(feature = "h7"))]
+        let num_data_write = len_write as u16;
+
+        #[cfg(feature = "h7")]
+        let num_data_read = len_read as u32;
+        #[cfg(not(feature = "h7"))]
+        let num_data_read = len_read as u16;
+
+        // Be careful - order of enabling Rx and Tx may matter, along with other things like when we
+        // enable the channels, and the SPI periph.
+        #[cfg(not(feature = "h7"))]
+        self.regs.cr2.modify(|_, w| w.rxdmaen().set_bit());
+        #[cfg(feature = "h7")]
+        self.regs.cfg1.modify(|_, w| w.rxdmaen().set_bit());
+
+        dma.cfg_channel(
+            channel_write,
+            periph_addr_write,
+            ptr_write as u32,
+            num_data_write,
+            dma::Direction::ReadFromMem,
+            dma::DataSize::S8,
+            dma::DataSize::S8,
+            channel_cfg_write,
+        );
+
+        dma.cfg_channel(
+            channel_read,
+            periph_addr_read,
+            ptr_read as u32,
+            num_data_read,
+            dma::Direction::ReadFromPeriph,
+            dma::DataSize::S8,
+            dma::DataSize::S8,
+            channel_cfg_read,
+        );
+
+        #[cfg(not(feature = "h7"))]
+        self.regs.cr2.modify(|_, w| w.txdmaen().set_bit());
+        #[cfg(feature = "h7")]
+        self.regs.cfg1.modify(|_, w| w.txdmaen().set_bit());
+
+        self.regs.cr1.modify(|_, w| w.spe().set_bit());
+    }
 
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Stop a DMA transfer. Stops the channel, and disables the `txdmaen` and `rxdmaen` bits.
@@ -676,7 +792,7 @@ where
         // 1. Disable DMA streams for Tx and Rx in the DMA registers, if the streams are used.
         dma.stop(channel);
         // 2. Disable the SPI by following the SPI disable procedure:
-        // self.disable(); // todo: This probably isn't required.
+        // self.disable();
         // 3. Disable DMA Tx and Rx buffers by clearing the TXDMAEN and RXDMAEN bits in the
         // SPI_CR2 register, if DMA Tx and/or DMA Rx are used.
         self.regs.cr2.modify(|_, w| {
@@ -702,7 +818,7 @@ where
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> FullDuplex<u8> for Spi<R>
 where
-    R: Deref<Target = pac::spi1::RegisterBlock> + DmaPeriph + RccPeriph,
+    R: Deref<Target = pac::spi1::RegisterBlock> + RccPeriph,
 {
     type Error = Error;
 
@@ -718,13 +834,13 @@ where
 #[cfg(feature = "embedded-hal")]
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> embedded_hal::blocking::spi::transfer::Default<u8> for Spi<R> where
-    R: Deref<Target = pac::spi1::RegisterBlock> + DmaPeriph + RccPeriph
+    R: Deref<Target = pac::spi1::RegisterBlock> + RccPeriph
 {
 }
 
 #[cfg(feature = "embedded-hal")]
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> embedded_hal::blocking::spi::write::Default<u8> for Spi<R> where
-    R: Deref<Target = pac::spi1::RegisterBlock> + DmaPeriph + RccPeriph
+    R: Deref<Target = pac::spi1::RegisterBlock> + RccPeriph
 {
 }
