@@ -10,8 +10,12 @@
 use crate::{
     clocks::Clocks,
     pac::{self, RCC},
-    util::{BaudPeriph, DmaPeriph, RccPeriph},
+    util::{BaudPeriph, RccPeriph},
 };
+
+#[cfg(any(feature = "f3", feature = "l4"))]
+use crate::util::DmaPeriph;
+
 use core::ops::Deref;
 
 use cortex_m::interrupt::free;
@@ -90,6 +94,16 @@ pub enum OverSampling {
     O8 = 1,
 }
 
+#[derive(Clone, Copy, PartialEq)]
+pub enum IrdaMode {
+    /// "IrDA mode disabled
+    None,
+    /// "IrDA SIR rx/tx enabled in 'normal' mode"
+    Normal,
+    /// "IrDA SIR 'low-power' mode"
+    LowPower,
+}
+
 #[cfg(not(feature = "f4"))]
 #[derive(Clone, Copy)]
 /// The type of USART interrupt to configure. Reference the USART_ISR register.
@@ -112,10 +126,16 @@ pub enum UsartInterrupt {
 
 /// Configuration for Usart. Can be used with default::Default.
 pub struct UsartConfig {
+    /// Word length. Defaults to 8-bits.
     pub word_len: WordLen,
+    /// Stop bits: Defaults to 1.
     pub stop_bits: StopBits,
+    /// Oversampling rate. Defaults to 16x.
     pub oversampling: OverSampling,
+    /// Enable or disable parity control. Defaults to disabled.
     pub parity: Parity,
+    /// IrDA mode: Enables this protocol, which is used to communicate with IR devices.
+    pub irda_mode: IrdaMode,
 }
 
 impl Default for UsartConfig {
@@ -125,6 +145,7 @@ impl Default for UsartConfig {
             stop_bits: StopBits::S1,
             oversampling: OverSampling::O16,
             parity: Parity::Disabled,
+            irda_mode: IrdaMode::None,
         }
     }
 }
@@ -138,7 +159,8 @@ pub struct Usart<R> {
 
 impl<R> Usart<R>
 where
-    R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
+    // R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
+    R: Deref<Target = pac::usart1::RegisterBlock> + RccPeriph + BaudPeriph,
 {
     /// Initialize a U[s]ART peripheral, including configuration register writes, and enabling and
     /// resetting its RCC peripheral clock. `baud` is the baud rate, in bytes-per-second.
@@ -207,6 +229,29 @@ where
             w.re().set_bit()
         });
 
+        match result.config.irda_mode {
+            // See G4 RM, section 37.5.18: USART IrDA SIR ENDEC block
+            // " IrDA mode is selected by setting the IREN bit in the USART_CR3 register. In IrDA mode,
+            // the following bits must be kept cleared:
+            // • LINEN, STOP and CLKEN bits in the USART_CR2 register,
+            IrdaMode::None => (),
+            _ => {
+                result.regs.cr2.modify(|_, w| unsafe {
+                    w.linen().clear_bit();
+                    w.stop().bits(0);
+                    w.clken().clear_bit()
+                });
+
+                // • SCEN and HDSEL bits in the USART_CR3 register."
+                result.regs.cr3.modify(|_, w| {
+                    w.scen().clear_bit();
+                    w.hdsel().clear_bit();
+                    w.irlp().bit(result.config.irda_mode == IrdaMode::LowPower);
+                    w.iren().set_bit()
+                });
+            }
+        }
+
         result
     }
 
@@ -409,9 +454,9 @@ where
         R::read_sel(dma);
 
         #[cfg(feature = "h7")]
-        let len = len as u32;
+        let num_data = len as u32;
         #[cfg(not(feature = "h7"))]
-        let len = len as u16;
+        let num_data = len as u16;
 
         dma.cfg_channel(
             channel,
@@ -424,7 +469,7 @@ where
             // RXNE event.
             ptr as u32,
             // 3. Configure the total number of bytes to be transferred to the DMA control register.
-            len,
+            num_data,
             dma::Direction::ReadFromPeriph,
             dma::DataSize::S8,
             dma::DataSize::S8,
@@ -564,7 +609,7 @@ pub enum Error {
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> Read<u8> for Usart<R>
 where
-    R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
+    R: Deref<Target = pac::usart1::RegisterBlock> + RccPeriph + BaudPeriph,
 {
     type Error = Error;
 
@@ -595,7 +640,7 @@ where
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> Write<u8> for Usart<R>
 where
-    R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
+    R: Deref<Target = pac::usart1::RegisterBlock> + RccPeriph + BaudPeriph,
 {
     type Error = Error;
 
@@ -645,7 +690,7 @@ where
 // #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<R> blocking::serial::Write<u8> for Usart<R>
 where
-    R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
+    R: Deref<Target = pac::usart1::RegisterBlock> + RccPeriph + BaudPeriph,
 {
     type Error = Error;
 
